;***************************************************************************************
;   throttle_timer.asm         DX6i Throttle Timer             
;***************************************************************************************
;                     Bruce Abbott   bruce.abbott@xtra.co.nz
;
;                     for Microchip 12F615, 12F675 or 12F683
;
; Automatically starts and stops the timer when the throttle stick is moved above 
; or below 20% throttle. Allows you to keep track of motor run time without having 
; to press a button or pull a switch! 
;
; NOTE1: Radio must be programmed to activate the timer on the throttle cut button.
;        Trainer switch will be free for binding, range test and trainer functions.
;
; NOTE2: The throttle cut function will be inactive. If you want to keep the
;        throttle cut function for a specific model then you must deactivate 
;        this circiut by holding down the throttle cut button when turning on 
;        the radio.
;
; NOTE3: You _could_ connect to the trainer switch instead of the throttle cut 
;        button, but other trainer switch functions may be affected. Using the 
;        throttle cut button for controlling the timer is the best option for 
;        electric powered models, as the button is otherwise redundant.
;
;=======================================================================================
; Wiring: 
;
; pin 1 = +3.3V (throttle stick pot RED wire)
; pin 2 = (optional) status LED cathode. Anode to pin 1 via 1k resistor 
; pin 3 = connected to throttle stick pot wiper (ORANGE wire)
; pin 4 = GND (connect to pin 8)
; pin 5 = connected to throttle cut button 'hot' side (GREY wire)
; pin 6 = n/c
; pin 7 = n/c
; pin 8 = GND (throttle stick pot BLACK wire)
;
; 0.1uF caramic capacitor connected between pin 1 and pin 8
;
; The PIC chip may be mounted on a small perf board, or simply wired 'dead bug' 
; style. An 8 pin IC socket is recommended, in case the chip needs to be removed
; for reprograming etc. 
;
; To connect to the pot wires, carefully strip back a small amount of insulation on 
; each wire in the middle of the throttle pot cable, then solder the wires to the IC 
; socket.
;
; One extra wire must be added, connecting from pin 5 of the PIC chip to the throttle 
; cut button's active terminal (which already has a GREY wire going to it). 
;
; The circuit should be wrapped in insulation tape to prevent it from shorting out 
; against anything else inside the radio!        
;
;---------------------------------------------------------------------------------------
;                             Summary of Changes
;
; 2012/06/04 V0.0 - Created
; 2012/05/06 V0.1 - disabled if throttle cut button pressed at startup  
;
; --------------------------------------------------------------------------------------

#DEFINE version  " 0 . 1"

; constants

#DEFINE	MINSTICK   19	; 0.24V  may vary depending on pot calibration! 
#DEFINE MAXSTICK  240	; 3.10V  may vary depending on pot calibration! 

#DEFINE LOWSTICK   52	; timer stop below 15% throttle = (240-19)*15%+19
#DEFINE HIGHSTICK  63	; timer start above 20% throttle = (240-19)*20%+19

; processor defines

;#DEFINE __12F615	; MPLAB users should use menu <Configure/Select Device>
;#DEFINE __12F675
;#DEFINE __12F683

	ifdef	  __12F615
        PROCESSOR PIC12F615
        INCLUDE   <P12F615.inc>
#DEFINE	RAMSTART  0x40
	__CONFIG  _MCLRE_OFF&_CP_OFF&_WDT_ON&_IOSCFS_4MHZ&_INTRC_OSC_NOCLKOUT
	endif

	ifdef	  __12F675
        PROCESSOR PIC12F675
        INCLUDE   <P12F675.inc>
#DEFINE	RAMSTART  0x20
	__CONFIG  _MCLRE_OFF&_CP_OFF&_WDT_ON&_BODEN_ON&_INTRC_OSC_NOCLKOUT
	endif

	ifdef	  __12F683
        PROCESSOR PIC12F683
        INCLUDE   <P12F683.inc>
#DEFINE	RAMSTART  0x20
	__CONFIG  _MCLRE_OFF&_CP_OFF&_WDT_ON&_BOD_ON&_INTRC_OSC_NOCLKOUT
	endif

	errorlevel 0,-305,-302
        radix     dec	; decimal numbers

; Bit definitions for the GPIO register and the TRIS register

#DEFINE OUT0     0    ; pin 7  (GP0) 
#DEFINE	OUT1	 1    ; pin 6  (GP1) 
#DEFINE SWITCH	 2    ; pin 5  (GP2) Connected to Throttle Cut switch
#DEFINE IN3	 3    ; pin 4  (GP3) 
#DEFINE	POT	 4    ; pin 3  (GP4) throttle pot, input to A/D
#DEFINE LED	 5    ; pin 2  (GP5) status LED (optional)


#DEFINE TrisBits ((1<<SWITCH)|(1<<POT)) ; input pins


; -------------------- OPTIONS ------------------------
;   weak pullups enabled
;   Timer 0 source internal
;   Prescaler to Timer 0, divide by 256.

#DEFINE OptionBits B'00000111'


;=====================================================================================
; Macro for generating short time delays
;
NO_OP           MACRO   count
NO_OP_COUNT     SET     count
                WHILE   NO_OP_COUNT>1
		goto	$+1		; 2 clocks
NO_OP_COUNT     SET     NO_OP_COUNT-2
                ENDW
		IF	NO_OP_COUNT
		nop			; 1 clock
		ENDIF
                ENDM

; ====================================================================================
; RAM Variable Definitions
;
        cblock  RAMSTART
	Flags
	PotVoltsf	; Potentiometer Voltage (fractional part)
	PotVolts	; Potentiometer Voltage 0~255 = 0~5V
	Temp1
	Temp2
	endc


; flag values
;
#DEFINE WATCH		0	; Watchdog timed out
#DEFINE STICKUP		1	; throttle stick was up 
#DEFINE	STICKDN		2	; throttle stick was down

;*************************************************************************************
;                                Code
;
	ORG	0
	goto	ColdStart	; jump to power-on startup

;------------------------------ version string ---------------------------------------

	org	8
	dw	" T T M R"
	ifdef	__12F615
	dw	" 6 1 5  "
	endif
	ifdef	__12F675
	dw	" 6 7 5  "
	endif
	ifdef	__12F683
	dw	" 6 8 3  "
	endif
	dw	" V"
	dw	version

;=====================================================================================
;                Startup from Power On or Watchdog Timeout
;	
ColdStart:
	bcf	Flags,WATCH
	btfss	STATUS,NOT_TO		; copy Watchdog timeout flag
	bsf	Flags,WATCH

; 12F683: set internal oscillator frequency
	ifdef	__12F683
	banksel	OSCCON
	movlw	b'01100001'		; 4MHz internal RC Oscillator
	movwf	OSCCON
	banksel	0
	endif

; 12F675: get oscillator calibration value and use it to fine-tune clock frequency.
	ifdef	__12F675
	banksel	OSCCAL			
	call	0x3ff			; get OSCCAL value
	movwf	OSCCAL			; set oscillator calibration
        banksel	0
	endif

; set options

	banksel OPTION_REG		 
        movlw   OptionBits
        movwf	OPTION_REG
	banksel	0

;====================================================================================
; initialise I/O registers

        clrf	GPIO			; all outputs low

	banksel	TRISIO			
        movlw	TrisBits
        movwf	TRISIO			; set I/O pin directions
	banksel	0			

	ifdef	__12F615
	movlw	b'00000000'
        movwf	CMCON0			; Comparator off
	movwf	CCP1CON			; CCP module off
	endif

	ifdef	__12F675
	movlw	b'00000111'
        movwf	CMCON			; Comparator off
	endif

	ifdef	__12F683
	movlw	b'00000111'
	movwf	CMCON0			; comparator off
	movlw	b'00000000'
	movwf	CCP1CON			; CCP module off	
	endif

; set up A/D convertor
	banksel	ANSEL
	movlw	b'00011000'	; clk/8, AN3 active
	movwf	ANSEL
	banksel	0
	movlw	b'10001101'	; right-justify, Vdd ref, input AN3, A/D on
	movwf	ADCON0

; CPU specific stuff done, now we can start the main program!

        goto	Main


;-----------------------------------------------------------------------------------
;                        Read Potentiometer Voltage
;-----------------------------------------------------------------------------------
;
; 64 x 10 bit A/D samples are summed to get 16 bits. Lower 8 bits are discarded.  
;
; output: PotVolts = 0~255 = 0~Vcc (3.3V) ~13mV resolution
;              
ReadPot:	
	movlw	64		; 64 x oversampling
	movwf	Temp1
	clrf	PotVoltsf	; no sample yet!  
	clrf	PotVolts
pot_read:
	bsf	ADCON0,GO	; start A/D conversion
pot_wait:
	btfsc	ADCON0,GO	; wait until A/D done
	goto	pot_wait
	banksel	ADRESL
	movf	ADRESL,w	; get A/D value low byte
	banksel	0
	addwf	PotVoltsf	; add to 16 bit accumulated result
	skpnc			; overflow? 
	incf	PotVolts	; carry to high byte
	movf	ADRESH,w	; get A/D value high byte
	addwf	PotVolts	; add to accumulated result	
	decfsz	Temp1		; next reading
	goto	pot_read
	return


;------------------------------------------------------------------------------------
;                                Switch On 
;------------------------------------------------------------------------------------
; Enable low output on switch pin to simulate pressing of thottle cut switch  
;
; NOTE: must set switch bit low here, because other read/modify/writes to GPIO 
;       register (when switch pin was an input) may have forced it high.  
;
SwitchOn:	
		bcf	GPIO,SWITCH	; ensure that switch output bit is low
		banksel	TRISIO
		bcf	TRISIO,SWITCH	; switch pin is now an output, forced low
		banksel	0
		return

;-----------------------------------------------------------------------------------
;                                 Switch Off 
;-----------------------------------------------------------------------------------
; Disable switch output
;
SwitchOff:	
		banksel	TRISIO
		bsf	TRISIO,SWITCH ; switch pin is now an input, floating high
		banksel	0
		return


;
;*********************** General purpose functions *********************************
;

;-----------------------------------------------------------------------------------
;                       11~266 Microsecond Delay
;-----------------------------------------------------------------------------------
; Input: w = delay time in microseconds -11
;
usdelay:	addlw	-4
		skpnc
		goto	usdelay
		sublw	-1
		addwf	PCL,f
		nop
		nop
		nop
		nop
		return


;------------------------------------------------------------------------------------
;                         1 millisecond Delay
;------------------------------------------------------------------------------------
; Input: w = delay time in microseconds
;
delay1ms:	; 1 millisecond delay
		movlw	-8	; preset countdown value to W
delayx4us:
		addlw	-1	; subtract 1
		btfsc	STATUS,C
		goto	delayx4us
		return


;------------------------------------------------------------------------------------
;                       Millisecond Delay Timer
;------------------------------------------------------------------------------------
; Input: W = number of milliseconds to wait (max 256mS)
;

dx1k:		movwf	Temp1
_dx1k1:		movlw	(1000-5)/5
		movwf	Temp2
_dx1k2:		clrwdt			; avoid watchdog timeout
		nop
		decfsz	Temp2		; wait 1mS
		goto	_dx1k2
		decfsz	Temp1
		goto	_dx1k1
		retlw	0


;*******************************************************************************
;				   Main
;*******************************************************************************


Main:		btfsc	Flags,WATCH	; did the watchdog timeout ?
        	goto    restart		; oops! try to keep going ...

		clrf	Flags		; clear all flags

restart:	bsf	GPIO,LED	; status LED on

		movlw	250		; wait 250mS for power to stabilise
		call	dx1k

		bcf	GPIO,LED	; status LED off

		btfss	GPIO,SWITCH	; throttle cut button held down?
		goto	killswitch	; yes, disable throttle timer

		bcf	Flags,STICKUP	; stick not up... 
		bcf	Flags,STICKDN	; ... or down! 

waitlow:	clrwdt			; reset watchdog timer
		call	ReadPot		; get pot position
		movlw	LOWSTICK
		subwf	PotVolts,w	; below low threshold?
		skpnc
		goto	waitlow		; no, wait until it is

		bsf	Flags,STICKDN	; stick is down

		bsf	GPIO,LED	; status LED off

MainLoop:	clrwdt			; reset watchdog timer
		call	ReadPot		; get pot position
		movlw	HIGHSTICK
		subwf	PotVolts,w	; above high threshold?
		skpnc
		goto	pothi		; yes
		movlw	LOWSTICK
		subwf	PotVolts,w	; no, below low threshold?
		skpc
		goto	potlow		; yes
		goto	done		; no, between high and low so do nothing

pothi:		bsf	Flags,STICKUP
		btfss	Flags,STICKDN
		goto	done
		bcf	Flags,STICKDN
		goto	pulsetimer	; trigger timer if stick was down 

potlow:		bsf	Flags,STICKDN
		btfss	Flags,STICKUP
		goto	done
		bcf	Flags,STICKUP	; trigger timer if stick was up

pulsetimer:	bcf	GPIO,LED	; status LED on
		call	SwitchOn
		movlw	250
		call	dx1k		; press throttle cut button for 0.25 seconds
		call	SwitchOff
		movlw	250
		call	dx1k		; delay after releasing button 

		bsf	GPIO,LED	; status LED off

done:		goto	MainLoop

; throttle kill button was pressed at startup, so disable throttle timer 

killswitch:	clrwdt
		goto	killswitch	; do nothing forever!


;------------------ Oscillator Calibration Subroutine -----------------------
;
; Only required if the original factory programmed value has been erased.
;
; NOTE: You should read the factory programmed value and record it, BEFORE 
;       programming your 12F675!

		ifdef	__12F675
		org	0x3ff
		retlw	0x80	; replace with OSCCAL value for your PIC
		endif

		END


